
#include "g_local.h"
#include "m_player.h"
// COVEN:  Include new header files.
#include "x_client.h"
#include "x_ctf.h"
#include "x_hud.h"
#include "x_jet.h"
#include "x_sound.h"
// COVEN



static	edict_t		*current_player;
static	gclient_t	*current_client;

static	vec3_t	forward, right, up;
float	xyspeed;

float	bobmove;
int		bobcycle;		// odd cycles are right foot going forward
float	bobfracsin;		// sin(bobfrac*M_PI)

/*
===============
SV_CalcRoll

===============
*/
float SV_CalcRoll (vec3_t angles, vec3_t velocity)
{
	float	sign;
	float	side;
	float	value;
	
	side = DotProduct (velocity, right);
	sign = side < 0 ? -1 : 1;
	side = fabs(side);
	
	value = sv_rollangle->value;

	if (side < sv_rollspeed->value)
		side = side * value / sv_rollspeed->value;
	else
		side = value;
	
	return side*sign;
	
}


/*
===============
P_DamageFeedback

Handles color blends and view kicks
===============
*/
void P_DamageFeedback (edict_t *player)
{
	gclient_t	*client;
	float	side;
	float	realcount, count, kick;
	vec3_t	v;
//        int             r, l;
	static	vec3_t	power_color = {0.0, 1.0, 0.0};
	static	vec3_t	acolor = {1.0, 1.0, 1.0};
	static	vec3_t	bcolor = {1.0, 0.0, 0.0};

	client = player->client;

	// flash the backgrounds behind the status numbers
	client->ps.stats[STAT_FLASHES] = 0;
	if (client->damage_blood)
		client->ps.stats[STAT_FLASHES] |= 1;
	if (client->damage_armor && !(player->flags & FL_GODMODE) && (client->invincible_framenum <= level.framenum))
		client->ps.stats[STAT_FLASHES] |= 2;

	// total points of damage shot at the player this frame
	count = (client->damage_blood + client->damage_armor + client->damage_parmor);
	if (count == 0)
		return;		// didn't take any damage

	// start a pain animation if still in the player model
// COVEN:  Player may not be 255.
        if ((client->anim_priority < ANIM_PAIN) && (Coven_IsPlayer(player)))
	{
		client->anim_priority = ANIM_PAIN;
                if (player->battlesuit)
                {       // BATTLESUIT
                        if ((client->ps.pmove.pm_flags & PMF_DUCKED) || (player->armoured))
                        {       // CROUCHING or ARMOURED
                                player->s.frame = player->crouch_pain_anim_start-1;
				client->anim_end = player->crouch_pain_anim_end;
			}
                        else if (player->waterlevel && !player->groundentity && !Jet_Active(player))
                        {       // SWIMMING
                                player->s.frame = player->swim_pain_anim_start-1;
                                client->anim_end = player->swim_pain_anim_end;
                        }
                        else
                        {       // STANDING
                                // Battlesuits only have one pain scene.
				player->s.frame = player->pain_anim_start-1;
				client->anim_end = player->pain_anim_end;
			}
                }
                else
                {       // PPM
                        if (client->ps.pmove.pm_flags & PMF_DUCKED)
                        {
                                player->s.frame = FRAME_crpain1-1;
                                client->anim_end = FRAME_crpain4;
                        }
                        else
                        {
                                static int              i;

                                i = (i+1)%3;
                                switch (i)
                                {
                                case 0:
                                        player->s.frame = FRAME_pain101-1;
                                        client->anim_end = FRAME_pain104;
                                        break;
                                case 1:
                                        player->s.frame = FRAME_pain201-1;
                                        client->anim_end = FRAME_pain204;
                                        break;
                                case 2:
                                        player->s.frame = FRAME_pain301-1;
                                        client->anim_end = FRAME_pain304;
                                        break;
                                }
                        }
                }
	}
// COVEN

	realcount = count;
	if (count < 10)
		count = 10;	// always make a visible effect

	// play an apropriate pain sound
	if ((level.time > player->pain_debounce_time) && !(player->flags & FL_GODMODE) && (client->invincible_framenum <= level.framenum))
	{
// COVEN:  Scream if on fire!
                player->pain_debounce_time = level.time + 0.7;
                if (player->burnout > level.time)
                        Coven_SfxBurn (player);
                else
                        Coven_SfxPain (player);
// COVEN
	}

	// the total alpha of the blend is always proportional to count
	if (client->damage_alpha < 0)
		client->damage_alpha = 0;
	client->damage_alpha += count*0.01;
	if (client->damage_alpha < 0.2)
		client->damage_alpha = 0.2;
	if (client->damage_alpha > 0.6)
		client->damage_alpha = 0.6;		// don't go too saturated

	// the color of the blend will vary based on how much was absorbed
	// by different armors
	VectorClear (v);
	if (client->damage_parmor)
		VectorMA (v, (float)client->damage_parmor/realcount, power_color, v);
	if (client->damage_armor)
		VectorMA (v, (float)client->damage_armor/realcount,  acolor, v);
	if (client->damage_blood)
		VectorMA (v, (float)client->damage_blood/realcount,  bcolor, v);
	VectorCopy (v, client->damage_blend);


	//
	// calculate view angle kicks
	//
	kick = abs(client->damage_knockback);
	if (kick && player->health > 0)	// kick of 0 means no view adjust at all
	{
		kick = kick * 100 / player->health;

		if (kick < count*0.5)
			kick = count*0.5;
		if (kick > 50)
			kick = 50;

		VectorSubtract (client->damage_from, player->s.origin, v);
		VectorNormalize (v);
		
		side = DotProduct (v, right);
		client->v_dmg_roll = kick*side*0.3;
		
		side = -DotProduct (v, forward);
		client->v_dmg_pitch = kick*side*0.3;

		client->v_dmg_time = level.time + DAMAGE_TIME;
	}

	//
	// clear totals
	//
	client->damage_blood = 0;
	client->damage_armor = 0;
	client->damage_parmor = 0;
	client->damage_knockback = 0;
}




/*
===============
SV_CalcViewOffset

Auto pitching on slopes?

  fall from 128: 400 = 160000
  fall from 256: 580 = 336400
  fall from 384: 720 = 518400
  fall from 512: 800 = 640000
  fall from 640: 960 = 

  damage = deltavelocity*deltavelocity  * 0.0001

===============
*/
void SV_CalcViewOffset (edict_t *ent)
{
	float		*angles;
	float		bob;
	float		ratio;
	float		delta;
	vec3_t		v;


//===================================

	// base angles
	angles = ent->client->ps.kick_angles;

	// if dead, fix the angle and don't add any kick
	if (ent->deadflag)
	{
		VectorClear (angles);

		ent->client->ps.viewangles[ROLL] = 40;
		ent->client->ps.viewangles[PITCH] = -15;
		ent->client->ps.viewangles[YAW] = ent->client->killer_yaw;
	}
	else
	{
		// add angles based on weapon kick

		VectorCopy (ent->client->kick_angles, angles);

		// add angles based on damage kick

		ratio = (ent->client->v_dmg_time - level.time) / DAMAGE_TIME;
		if (ratio < 0)
		{
			ratio = 0;
			ent->client->v_dmg_pitch = 0;
			ent->client->v_dmg_roll = 0;
		}
		angles[PITCH] += ratio * ent->client->v_dmg_pitch;
		angles[ROLL] += ratio * ent->client->v_dmg_roll;

		// add pitch based on fall kick

		ratio = (ent->client->fall_time - level.time) / FALL_TIME;
		if (ratio < 0)
			ratio = 0;
		angles[PITCH] += ratio * ent->client->fall_value;

		// add angles based on velocity

		delta = DotProduct (ent->velocity, forward);
		angles[PITCH] += delta*run_pitch->value;
		
		delta = DotProduct (ent->velocity, right);
		angles[ROLL] += delta*run_roll->value;

		// add angles based on bob

		delta = bobfracsin * bob_pitch->value * xyspeed;
		if (ent->client->ps.pmove.pm_flags & PMF_DUCKED)
			delta *= 6;		// crouching
		angles[PITCH] += delta;
		delta = bobfracsin * bob_roll->value * xyspeed;
		if (ent->client->ps.pmove.pm_flags & PMF_DUCKED)
			delta *= 6;		// crouching
		if (bobcycle & 1)
			delta = -delta;
		angles[ROLL] += delta;
	}

//===================================

	// base origin

	VectorClear (v);

	// add view height

	v[2] += ent->viewheight;

	// add fall height

	ratio = (ent->client->fall_time - level.time) / FALL_TIME;
	if (ratio < 0)
		ratio = 0;
	v[2] -= ratio * ent->client->fall_value * 0.4;

	// add bob height

	bob = bobfracsin * xyspeed * bob_up->value;
	if (bob > 6)
		bob = 6;
	//gi.DebugGraph (bob *2, 255);
	v[2] += bob;

	// add kick offset

	VectorAdd (v, ent->client->kick_origin, v);

	// absolutely bound offsets
	// so the view can never be outside the player box

	if (v[0] < -14)
		v[0] = -14;
	else if (v[0] > 14)
		v[0] = 14;
	if (v[1] < -14)
		v[1] = -14;
	else if (v[1] > 14)
		v[1] = 14;
	if (v[2] < -22)
		v[2] = -22;
	else if (v[2] > 30)
		v[2] = 30;

	VectorCopy (v, ent->client->ps.viewoffset);
}

/*
==============
SV_CalcGunOffset
==============
*/
void SV_CalcGunOffset (edict_t *ent)
{
	int		i;
	float	delta;

	// gun angles from bobbing
	ent->client->ps.gunangles[ROLL] = xyspeed * bobfracsin * 0.005;
	ent->client->ps.gunangles[YAW] = xyspeed * bobfracsin * 0.01;
	if (bobcycle & 1)
	{
		ent->client->ps.gunangles[ROLL] = -ent->client->ps.gunangles[ROLL];
		ent->client->ps.gunangles[YAW] = -ent->client->ps.gunangles[YAW];
	}

	ent->client->ps.gunangles[PITCH] = xyspeed * bobfracsin * 0.005;

	// gun angles from delta movement
	for (i=0 ; i<3 ; i++)
	{
		delta = ent->client->oldviewangles[i] - ent->client->ps.viewangles[i];
		if (delta > 180)
			delta -= 360;
		if (delta < -180)
			delta += 360;
		if (delta > 45)
			delta = 45;
		if (delta < -45)
			delta = -45;
		if (i == YAW)
			ent->client->ps.gunangles[ROLL] += 0.1*delta;
		ent->client->ps.gunangles[i] += 0.2 * delta;
	}

	// gun height
	VectorClear (ent->client->ps.gunoffset);
//	ent->ps->gunorigin[2] += bob;

	// gun_x / gun_y / gun_z are development tools
	for (i=0 ; i<3 ; i++)
	{
		ent->client->ps.gunoffset[i] += forward[i]*(gun_y->value);
		ent->client->ps.gunoffset[i] += right[i]*gun_x->value;
		ent->client->ps.gunoffset[i] += up[i]* (-gun_z->value);
	}
}


/*
=============
SV_AddBlend
=============
*/
void SV_AddBlend (float r, float g, float b, float a, float *v_blend)
{
	float	a2, a3;

	if (a <= 0)
		return;
	a2 = v_blend[3] + (1-v_blend[3])*a;	// new total alpha
	a3 = v_blend[3]/a2;		// fraction of color from old

	v_blend[0] = v_blend[0]*a3 + r*(1-a3);
	v_blend[1] = v_blend[1]*a3 + g*(1-a3);
	v_blend[2] = v_blend[2]*a3 + b*(1-a3);
	v_blend[3] = a2;
}


/*
=============
SV_CalcBlend
=============
*/
void SV_CalcBlend (edict_t *ent)
{
	int		contents;
	vec3_t	vieworg;
	int		remaining;

	ent->client->ps.blend[0] = ent->client->ps.blend[1] = 
		ent->client->ps.blend[2] = ent->client->ps.blend[3] = 0;

	// add for contents
	VectorAdd (ent->s.origin, ent->client->ps.viewoffset, vieworg);
	contents = gi.pointcontents (vieworg);
	if (contents & (CONTENTS_LAVA|CONTENTS_SLIME|CONTENTS_WATER) )
		ent->client->ps.rdflags |= RDF_UNDERWATER;
	else
		ent->client->ps.rdflags &= ~RDF_UNDERWATER;

	if (contents & (CONTENTS_SOLID|CONTENTS_LAVA))
		SV_AddBlend (1.0, 0.3, 0.0, 0.6, ent->client->ps.blend);
	else if (contents & CONTENTS_SLIME)
		SV_AddBlend (0.0, 0.1, 0.05, 0.6, ent->client->ps.blend);
	else if (contents & CONTENTS_WATER)
		SV_AddBlend (0.5, 0.3, 0.2, 0.4, ent->client->ps.blend);

	// add for powerups
// COVEN:  Check for jetpack fuel.
        if (Jet_Active(ent))
        {
                int     fuel = ent->client->pers.inventory[ITEM_INDEX(FindItem ("Fuel"))];

                if ((level.framenum - ent->client->jet_framenum) >= 10)
                {
                        if (fuel)
                        {       if (! ( (int)dmflags->value & DF_INFINITE_AMMO ) )
                                        ent->client->pers.inventory[ITEM_INDEX(FindItem ("Fuel"))]--;
                                ent->client->jet_framenum = level.framenum;
                        }
                        else
                                ent->client->jet_framenum = 0;
                }

                if ( ((int)level.framenum % 6) == 0 )
                        gi.sound (ent, CHAN_AUTO, gi.soundindex("hover/hovidle1.wav"), 0.9, ATTN_NORM, 0);
        }
// COVEN
	if (ent->client->quad_framenum > level.framenum)
	{
		remaining = ent->client->quad_framenum - level.framenum;
		if (remaining == 30)	// beginning to fade
			gi.sound(ent, CHAN_ITEM, gi.soundindex("items/damage2.wav"), 1, ATTN_NORM, 0);
		if (remaining > 30 || (remaining & 4) )
			SV_AddBlend (0, 0, 1, 0.08, ent->client->ps.blend);
	}
	else if (ent->client->invincible_framenum > level.framenum)
	{
		remaining = ent->client->invincible_framenum - level.framenum;
		if (remaining == 30)	// beginning to fade
			gi.sound(ent, CHAN_ITEM, gi.soundindex("items/protect2.wav"), 1, ATTN_NORM, 0);
		if (remaining > 30 || (remaining & 4) )
			SV_AddBlend (1, 1, 0, 0.08, ent->client->ps.blend);
	}
	else if (ent->client->enviro_framenum > level.framenum)
	{
		remaining = ent->client->enviro_framenum - level.framenum;
		if (remaining == 30)	// beginning to fade
			gi.sound(ent, CHAN_ITEM, gi.soundindex("items/airout.wav"), 1, ATTN_NORM, 0);
		if (remaining > 30 || (remaining & 4) )
			SV_AddBlend (0, 1, 0, 0.08, ent->client->ps.blend);
	}
	else if (ent->client->breather_framenum > level.framenum)
	{
		remaining = ent->client->breather_framenum - level.framenum;
		if (remaining == 30)	// beginning to fade
			gi.sound(ent, CHAN_ITEM, gi.soundindex("items/airout.wav"), 1, ATTN_NORM, 0);
		if (remaining > 30 || (remaining & 4) )
			SV_AddBlend (0.4, 1, 0.4, 0.04, ent->client->ps.blend);
	}

	// add for damage
	if (ent->client->damage_alpha > 0)
		SV_AddBlend (ent->client->damage_blend[0],ent->client->damage_blend[1]
		,ent->client->damage_blend[2], ent->client->damage_alpha, ent->client->ps.blend);

	if (ent->client->bonus_alpha > 0)
		SV_AddBlend (0.85, 0.7, 0.3, ent->client->bonus_alpha, ent->client->ps.blend);

// COVEN:  Blend in flash if applicable.
/*
// DISABLED!  Uncomment if we want nukes or similar weapons.
        if (ent->client->flash_alpha > 0)
                SV_AddBlend (1, 1, 1, ent->client->flash_alpha, ent->client->ps.blend);
*/
// COVEN

	// drop the damage value
	ent->client->damage_alpha -= 0.06;
	if (ent->client->damage_alpha < 0)
		ent->client->damage_alpha = 0;

	// drop the bonus value
	ent->client->bonus_alpha -= 0.1;
	if (ent->client->bonus_alpha < 0)
		ent->client->bonus_alpha = 0;

// COVEN:  Drop flash value.
/*
// DISABLED!  Uncomment if we want nukes or similar weapons.
        ent->client->flash_alpha -= ent->client->flash_fade;
        if (ent->client->flash_alpha < 0)
        {
                ent->client->flash_alpha = 0;
                ent->client->flash_fade = 0;
        }
*/
// COVEN
}


/*
=================
P_FallingDamage
=================
*/
void P_FallingDamage (edict_t *ent)
{
	float	delta;
	int		damage;
	vec3_t	dir;

// COVEN:  Player may not be 255.
        if (!Coven_IsPlayer(ent))
// COVEN
		return;		// not in the player model

	if (ent->movetype == MOVETYPE_NOCLIP)
		return;

// COVEN:  Active jetpack negates falling damage.
        if (Jet_Active (ent))
                return;
// COVEN

	if ((ent->client->oldvelocity[2] < 0) && (ent->velocity[2] > ent->client->oldvelocity[2]) && (!ent->groundentity))
	{
		delta = ent->client->oldvelocity[2];
	}
	else
	{
		if (!ent->groundentity)
			return;
		delta = ent->velocity[2] - ent->client->oldvelocity[2];
	}
	delta = delta*delta * 0.0001;

	// never take falling damage if completely underwater
	if (ent->waterlevel == 3)
		return;
	if (ent->waterlevel == 2)
		delta *= 0.25;
	if (ent->waterlevel == 1)
		delta *= 0.5;

// COVEN:  Falling damage resistance.
        // Riding on grapple softens impact.
        if (ent->hook)
                if (!ent->hook->solid)
                        delta *= 0.5;

        // Light suits resist falling damage.
        if ((ent->battlesuit == SUIT_MERCURY) || (ent->battlesuit == SUIT_ZEPPELIN))
                delta *= 0.75;
// COVEN

	if (delta < 1)
		return;

	if (delta < 15)
	{
// COVEN:  Play AB suit sfx.
                if (ent->groundentity)
                        Coven_SfxFootstep (ent);
// COVEN
		return;
	}

	ent->client->fall_value = delta*0.5;
	if (ent->client->fall_value > 40)
		ent->client->fall_value = 40;
	ent->client->fall_time = level.time + FALL_TIME;

	if (delta > 30)
	{
		if (ent->health > 0)
		{
// COVEN:  Play AB suit sfx.
			if (delta >= 55)
                                Coven_SfxFallFar (ent);
			else
                                Coven_SfxFall (ent);
// COVEN
		}
		ent->pain_debounce_time = level.time;	// no normal pain sound
		damage = (delta-30)/2;
		if (damage < 1)
			damage = 1;
		VectorSet (dir, 0, 0, 1);

		if (!deathmatch->value || !((int)dmflags->value & DF_NO_FALLING) )
			T_Damage (ent, world, world, dir, ent->s.origin, vec3_origin, damage, 0, 0, MOD_FALLING);
	}
	else
	{
// COVEN:  Play thud sound only if onground.
                if (ent->groundentity)
                        ent->s.event = EV_FALLSHORT;
// COVEN
		return;
	}
}



/*
=============
P_WorldEffects
=============
*/
void P_WorldEffects (void)
{
	qboolean	breather;
	qboolean	envirosuit;
	int			waterlevel, old_waterlevel;

	if (current_player->movetype == MOVETYPE_NOCLIP)
	{
// COVEN:  Variable air capacity.
                current_player->air_finished = level.time + current_player->air_max; // don't need air
// COVEN
		return;
	}

	waterlevel = current_player->waterlevel;
	old_waterlevel = current_client->old_waterlevel;
	current_client->old_waterlevel = waterlevel;

	breather = current_client->breather_framenum > level.framenum;
	envirosuit = current_client->enviro_framenum > level.framenum;

	//
	// if just entered a water volume, play a sound
	//
	if (!old_waterlevel && waterlevel)
	{
		PlayerNoise(current_player, current_player->s.origin, PNOISE_SELF);
		if (current_player->watertype & CONTENTS_LAVA)
			gi.sound (current_player, CHAN_BODY, gi.soundindex("player/lava_in.wav"), 1, ATTN_NORM, 0);
		else if (current_player->watertype & CONTENTS_SLIME)
			gi.sound (current_player, CHAN_BODY, gi.soundindex("player/watr_in.wav"), 1, ATTN_NORM, 0);
		else if (current_player->watertype & CONTENTS_WATER)
			gi.sound (current_player, CHAN_BODY, gi.soundindex("player/watr_in.wav"), 1, ATTN_NORM, 0);
		current_player->flags |= FL_INWATER;

		// clear damage_debounce, so the pain sound will play immediately
		current_player->damage_debounce_time = level.time - 1;
	}

	//
	// if just completely exited a water volume, play a sound
	//
	if (old_waterlevel && ! waterlevel)
	{
		PlayerNoise(current_player, current_player->s.origin, PNOISE_SELF);
		gi.sound (current_player, CHAN_BODY, gi.soundindex("player/watr_out.wav"), 1, ATTN_NORM, 0);
		current_player->flags &= ~FL_INWATER;
	}

	//
	// check for head just going under water
	//
	if (old_waterlevel != 3 && waterlevel == 3)
	{
		gi.sound (current_player, CHAN_BODY, gi.soundindex("player/watr_un.wav"), 1, ATTN_NORM, 0);
	}

	//
	// check for head just coming out of water
	//
	if (old_waterlevel == 3 && waterlevel != 3)
	{
// COVEN:  Suits have different air capacity and sounds.
                if (!breather && !envirosuit)
                {
                        if (current_player->air_finished < level.time)
                        {       // gasp for air
                                Coven_SfxGasp (current_player, 1);
                                PlayerNoise(current_player, current_player->s.origin, PNOISE_SELF);
                        }
                        else if (current_player->air_finished < level.time + current_player->air_max - 1)
                        {       // just break surface
                                Coven_SfxGasp (current_player, 2);
                        }
		}
// COVEN
	}

	//
	// check for drowning
	//
	if (waterlevel == 3)
	{
		// breather or envirosuit give air
		if (breather || envirosuit)
		{
// COVEN:  Variable air capacity.
                        current_player->air_finished = level.time + current_player->air_max - 2;
// COVEN

			if (((int)(current_client->breather_framenum - level.framenum) % 25) == 0)
			{
				if (!current_client->breather_sound)
					gi.sound (current_player, CHAN_AUTO, gi.soundindex("player/u_breath1.wav"), 1, ATTN_NORM, 0);
				else
					gi.sound (current_player, CHAN_AUTO, gi.soundindex("player/u_breath2.wav"), 1, ATTN_NORM, 0);
				current_client->breather_sound ^= 1;
				PlayerNoise(current_player, current_player->s.origin, PNOISE_SELF);
				//FIXME: release a bubble?
			}
		}

		// if out of air, start drowning
		if (current_player->air_finished < level.time)
		{	// drown!
			if (current_player->client->next_drown_time < level.time 
				&& current_player->health > 0)
			{
				current_player->client->next_drown_time = level.time + 1;

				// take more damage the longer underwater
				current_player->dmg += 2;
				if (current_player->dmg > 15)
					current_player->dmg = 15;

				// play a gurp sound instead of a normal pain sound
// COVEN:  Play AB suit sfx.
				if (current_player->health <= current_player->dmg)
                                        Coven_SfxDrown (current_player);
				else
                                        Coven_SfxGurp (current_player);
// COVEN

				current_player->pain_debounce_time = level.time;

				T_Damage (current_player, world, world, vec3_origin, current_player->s.origin, vec3_origin, current_player->dmg, 0, DAMAGE_NO_ARMOR, MOD_WATER);
			}
		}
	}
	else
	{
// COVEN:  Variable air capacity.
                current_player->air_finished = level.time + current_player->air_max;
// COVEN
		current_player->dmg = 2;
	}

	//
	// check for sizzle damage
	//
	if (waterlevel && (current_player->watertype&(CONTENTS_LAVA|CONTENTS_SLIME)) )
	{
		if (current_player->watertype & CONTENTS_LAVA)
		{
			if (current_player->health > 0
				&& current_player->pain_debounce_time <= level.time
				&& current_client->invincible_framenum < level.framenum)
			{
// COVEN:  Use special burn sfx.
                                Coven_SfxBurn (current_player);
// COVEN
				current_player->pain_debounce_time = level.time + 1;
			}

			if (envirosuit)	// take 1/3 damage with envirosuit
				T_Damage (current_player, world, world, vec3_origin, current_player->s.origin, vec3_origin, 1*waterlevel, 0, 0, MOD_LAVA);
			else
				T_Damage (current_player, world, world, vec3_origin, current_player->s.origin, vec3_origin, 3*waterlevel, 0, 0, MOD_LAVA);
		}

		if (current_player->watertype & CONTENTS_SLIME)
		{
			if (!envirosuit)
			{	// no damage from slime with envirosuit
				T_Damage (current_player, world, world, vec3_origin, current_player->s.origin, vec3_origin, 1*waterlevel, 0, 0, MOD_SLIME);
			}
		}
	}
}


/*
===============
G_SetClientEffects
===============
*/
void G_SetClientEffects (edict_t *ent)
{
	int		pa_type;
	int		remaining;

	ent->s.effects = 0;
	ent->s.renderfx = 0;

	if (ent->health <= 0 || level.intermissiontime)
		return;

	if (ent->powerarmor_time > level.time)
	{
		pa_type = PowerArmorType (ent);
		if (pa_type == POWER_ARMOR_SCREEN)
		{
			ent->s.effects |= EF_POWERSCREEN;
		}
		else if (pa_type == POWER_ARMOR_SHIELD)
		{
			ent->s.effects |= EF_COLOR_SHELL;
			ent->s.renderfx |= RF_SHELL_GREEN;
		}
	}

// COVEN:  Zoid's CTF.
	CTFEffects(ent);
// COVEN

	if (ent->client->quad_framenum > level.framenum)
	{
		remaining = ent->client->quad_framenum - level.framenum;
		if (remaining > 30 || (remaining & 4) )
// COVEN:  Battlesuits use colored shells based on team for quad.
                {
                        ent->s.effects |= EF_COLOR_SHELL;
                        if (ent->client->resp.suitcolor == SKIN_PURPLE)
                                ent->s.renderfx |= (RF_SHELL_RED|RF_SHELL_BLUE);
                        else if (ent->client->resp.suitcolor == SKIN_GOLD)
                                ent->s.renderfx |= (RF_SHELL_RED|RF_SHELL_GREEN);
                        else if (ent->client->resp.suitcolor == SKIN_GREEN)
                                ent->s.renderfx |= RF_SHELL_GREEN;
                        else if (ent->client->resp.suitcolor == SKIN_RED)
                                ent->s.renderfx |= RF_SHELL_RED;
                        else
                                ent->s.renderfx |= RF_SHELL_BLUE;
                }
// COVEN
	}

	if (ent->client->invincible_framenum > level.framenum)
	{
		remaining = ent->client->invincible_framenum - level.framenum;
		if (remaining > 30 || (remaining & 4) )
// COVEN:  Battlesuits use a white shell for invullnerability.
                {
                        ent->s.effects |= EF_COLOR_SHELL;
                        ent->s.renderfx |= (RF_SHELL_RED|RF_SHELL_GREEN|RF_SHELL_BLUE);
                }
// COVEN
	}

// COVEN:  Friendly fire shell.
	if (ent->teamfire_framenum > level.framenum)
        {       // Flash white shell for friendly fire.
		ent->s.effects |= EF_COLOR_SHELL;
                ent->s.renderfx |= (RF_SHELL_RED|RF_SHELL_GREEN|RF_SHELL_BLUE);
        }
// COVEN

	// show cheaters!!!
	if (ent->flags & FL_GODMODE)
	{
		ent->s.effects |= EF_COLOR_SHELL;
		ent->s.renderfx |= (RF_SHELL_RED|RF_SHELL_GREEN|RF_SHELL_BLUE);
	}
}


/*
===============
G_SetClientEvent
===============
*/
void G_SetClientEvent (edict_t *ent)
{
// COVEN:  Play footsteps for ArmourBack suits.
        qboolean        stomp;

	if (ent->s.event)
		return;

        if (ent->battlesuit == SUIT_BULKHEAD)
        {       // Bulkhead rolls on treads.
                if (ent->groundentity && xyspeed > 0 && (level.framenum & 1))
                        if (ent->health > 0)
                                gi.sound (ent, CHAN_BODY, gi.soundindex(va("armour/bulkhead/more/roll%i.wav", ((rand() % 4) + 1) )), 1, ATTN_NORM, 0);
                return;
        }

        // Could make Rhino/SC stomp on frames Rrun2/SCrun1, but don't need to.

        // Check if footstep should be made.
        if ((!ent->groundentity) || (ent->client->ps.pmove.pm_flags & PMF_DUCKED))
                stomp = false;
        else if (xyspeed > 225)
                stomp = true;
        else if ((ent->battlesuit == SUIT_RHINO) || (ent->battlesuit == SUIT_SUGARCONNY))
                stomp = true;
        else
                stomp = false;

        if (stomp)
        {       // Play AB suit sfx.
		if ( (int)(current_client->bobtime+bobmove) != bobcycle )
                        Coven_SfxFootstep (ent);
	}
// COVEN
}

/*
===============
G_SetClientSound
===============
*/
void G_SetClientSound (edict_t *ent)
{
	char	*weap;

	if (ent->client->pers.game_helpchanged != game.helpchanged)
	{
		ent->client->pers.game_helpchanged = game.helpchanged;
		ent->client->pers.helpchanged = 1;
	}

	// help beep (no more than three times)
	if (ent->client->pers.helpchanged && ent->client->pers.helpchanged <= 3 && !(level.framenum&63) )
	{
		ent->client->pers.helpchanged++;
		gi.sound (ent, CHAN_VOICE, gi.soundindex ("misc/pc_up.wav"), 1, ATTN_STATIC, 0);
	}


	if (ent->client->pers.weapon)
		weap = ent->client->pers.weapon->classname;
	else
		weap = "";

// COVEN:  Reorder sound priority.
        if (ent->client->weapon_sound)
		ent->s.sound = ent->client->weapon_sound;
        else if (ent->waterlevel && (ent->watertype&(CONTENTS_LAVA|CONTENTS_SLIME)) )
		ent->s.sound = snd_fry;
        else if (strcmp(weap, "weapon_photoncannon") == 0)
		ent->s.sound = gi.soundindex("weapons/rg_hum.wav");
// COVEN
	else
		ent->s.sound = 0;
}

/*
===============
G_SetClientFrame
===============
*/
void G_SetClientFrame (edict_t *ent)
{
	gclient_t	*client;
	qboolean	duck, run;
// COVEN:  New swim variable.
        qboolean        swim;
// COVEN

// COVEN:  Player may not be 255.
        if (!Coven_IsPlayer(ent))
// COVEN
		return;		// not in the player model

	client = ent->client;

// COVEN:  Played is considered 'duck'ed if crouching or armoured.
        if ((client->ps.pmove.pm_flags & PMF_DUCKED) || (ent->armoured))
// COVEN
		duck = true;
	else
		duck = false;
	if (xyspeed)
		run = true;
	else
		run = false;

// COVEN:  ArmourBack player models have swim anims!
        if ((ent->battlesuit) && (ent->waterlevel) && (!ent->groundentity) && (!Jet_Active(ent)))
		swim = true;
	else
		swim = false;



	/* duck/dont duck if doing anything except dying */
	if (duck != client->anim_duck && client->anim_priority < ANIM_DEATH)	// < 5
		goto newanim;
	/* swim/dont swim if client is doing anything except dying */
        if (swim != client->anim_swim && client->anim_priority < ANIM_DEATH)    // < 5
                goto newanim;
	/* run/dont run if client is doing nothing */
	if (run != client->anim_run && client->anim_priority == ANIM_BASIC)		// == 0
		goto newanim;
	/* if client is not on the ground and is waving or just hanging out */
        if (!swim && !ent->groundentity && client->anim_priority <= ANIM_WAVE)                   // <= 1
                goto newanim;
// COVEN

	// check for stand/duck and stop/go transitions
	if (duck != client->anim_duck && client->anim_priority < ANIM_DEATH)
		goto newanim;
// COVEN:  Check if player is swimming.
        if (swim != client->anim_swim && client->anim_priority < ANIM_DEATH)    // < 5
                goto newanim;
	if (run != client->anim_run && client->anim_priority == ANIM_BASIC)
		goto newanim;
        if (!swim && !ent->groundentity && client->anim_priority <= ANIM_WAVE)                   // <= 1
                goto newanim;
// COVEN

	if(client->anim_priority == ANIM_REVERSE)
	{
		if(ent->s.frame > client->anim_end)
		{
			ent->s.frame--;
			return;
		}
	}
	else if (ent->s.frame < client->anim_end)
	{	// continue an animation
		ent->s.frame++;
		return;
	}

	if (client->anim_priority == ANIM_DEATH)
// COVEN:  Bulkhead explodes when death anim is complete.
        {
                if (ent->battlesuit == SUIT_BULKHEAD)
                {       if (ent->health > ent->gib_health)
                        {       ent->health = -999;
                                Coven_SuitGib (ent, 999, 0);
                                ent->client->anim_priority = ANIM_DEATH;
                                ent->client->anim_end = 0;
                        }
                }
		return;		// stay there
        }
// COVEN
// COVEN:  Use suit or PPM jump anims.
	if (client->anim_priority == ANIM_JUMP)
	{
		if (!ent->groundentity)
			return;		// retain leaping frame
		ent->client->anim_priority = ANIM_WAVE;
                if (ent->battlesuit)
		{       ent->s.frame = ent->jump_anim_start + 2;
			ent->client->anim_end = ent->jump_anim_end;
		}
		else
		{       ent->s.frame = FRAME_jump3;
			ent->client->anim_end = FRAME_jump6;
		}
		return;
	}
// COVEN

newanim:
	// return to either a running or standing frame
	client->anim_priority = ANIM_BASIC;
	client->anim_duck = duck;
	client->anim_run = run;
// COVEN:  Swimming.
	client->anim_swim = swim;
// COVEN

// COVEN:  Animate suit or PPM.
        if (swim)
        {       // SWIMMING
                ent->s.frame = ent->swim_anim_start;
                client->anim_end = ent->swim_anim_end;
        }
        else if (!ent->groundentity)
        {       // IN MID-AIR
		client->anim_priority = ANIM_JUMP;
                if (ent->battlesuit)
                {       if (ent->s.frame != ent->jump_anim_start + 1)
                                ent->s.frame = ent->jump_anim_start;
                        client->anim_end = ent->jump_anim_start + 1;
		}
		else
                {       if (ent->s.frame != FRAME_jump2)
				ent->s.frame = FRAME_jump1;
			client->anim_end = FRAME_jump2;
		}
	}
	else if (run)
        {       // MOVING
                if (duck)
		{       if (ent->battlesuit)
                        {       ent->s.frame = ent->crouch_move_anim_start;
				client->anim_end = ent->crouch_move_anim_end;
			}
			else
			{       ent->s.frame = FRAME_crwalk1;
				client->anim_end = FRAME_crwalk6;
			}
		}
		else
		{       if (ent->battlesuit)
			{       ent->s.frame = ent->run_anim_start;
				client->anim_end = ent->run_anim_end;
			}
			else
			{       ent->s.frame = FRAME_run1;
				client->anim_end = FRAME_run6;
			}
		}
	}
	else
        {       // STATIONARY
		if (duck)
		{       if (ent->battlesuit)
			{       ent->s.frame = ent->crouch_anim_start;
				client->anim_end = ent->crouch_anim_end;
			}
			else
			{       ent->s.frame = FRAME_crstnd01;
				client->anim_end = FRAME_crstnd19;
			}
		}
		else 
		{       if (ent->battlesuit)
			{       ent->s.frame = ent->stand_anim_start;
				client->anim_end = ent->stand_anim_end;
			}
			else
			{       ent->s.frame = FRAME_stand01;
				client->anim_end = FRAME_stand40;
			}
		}
        }
// COVEN
}


/*
=================
ClientEndServerFrame

Called for each player at the end of the server frame
and right after spawning
=================
*/
void ClientEndServerFrame (edict_t *ent)
{
	float	bobtime;
	int		i;

	current_player = ent;
	current_client = ent->client;

	//
	// If the origin or velocity have changed since ClientThink(),
	// update the pmove values.  This will happen when the client
	// is pushed by a bmodel or kicked by an explosion.
	// 
	// If it wasn't updated here, the view position would lag a frame
	// behind the body position when pushed -- "sinking into plats"
	//
	for (i=0 ; i<3 ; i++)
	{
		current_client->ps.pmove.origin[i] = ent->s.origin[i]*8.0;
		current_client->ps.pmove.velocity[i] = ent->velocity[i]*8.0;
	}

	//
	// If the end of unit layout is displayed, don't give
	// the player any normal movement attributes
	//
	if (level.intermissiontime)
	{
		// FIXME: add view drifting here?
		current_client->ps.blend[3] = 0;
		current_client->ps.fov = 90;
		G_SetStats (ent);
		return;
	}

	AngleVectors (ent->client->v_angle, forward, right, up);

	// burn from lava, etc
	P_WorldEffects ();

	//
	// set model angles from view angles so other things in
	// the world can tell which direction you are looking
	//
	if (ent->client->v_angle[PITCH] > 180)
		ent->s.angles[PITCH] = (-360 + ent->client->v_angle[PITCH])/3;
	else
		ent->s.angles[PITCH] = ent->client->v_angle[PITCH]/3;
	ent->s.angles[YAW] = ent->client->v_angle[YAW];
	ent->s.angles[ROLL] = 0;
	ent->s.angles[ROLL] = SV_CalcRoll (ent->s.angles, ent->velocity)*4;

	//
	// calculate speed and cycle to be used for
	// all cyclic walking effects
	//
	xyspeed = sqrt(ent->velocity[0]*ent->velocity[0] + ent->velocity[1]*ent->velocity[1]);

	if (xyspeed < 5)
	{
		bobmove = 0;
		current_client->bobtime = 0;	// start at beginning of cycle again
	}
	else if (ent->groundentity)
	{	// so bobbing only cycles when on ground
		if (xyspeed > 210)
			bobmove = 0.25;
		else if (xyspeed > 100)
			bobmove = 0.125;
		else
			bobmove = 0.0625;
	}
	
	bobtime = (current_client->bobtime += bobmove);

	if (current_client->ps.pmove.pm_flags & PMF_DUCKED)
		bobtime *= 4;

	bobcycle = (int)bobtime;
	bobfracsin = fabs(sin(bobtime*M_PI));

	// detect hitting the floor
	P_FallingDamage (ent);

	// apply all the damage taken this frame
	P_DamageFeedback (ent);

	// determine the view offsets
	SV_CalcViewOffset (ent);

	// determine the gun offsets
	SV_CalcGunOffset (ent);

	// determine the full screen color blend
	// must be after viewoffset, so eye contents can be
	// accurately determined
	// FIXME: with client prediction, the contents
	// should be determined by the client
	SV_CalcBlend (ent);

	// chase cam stuff
	if (ent->client->resp.spectator)
// COVEN:  Don't show crosshairs and id in spectator mode.
        {
		G_SetSpectatorStats(ent);
                ent->client->ps.stats[STAT_CROSSHAIRS] = 0;
                Coven_ClearID (ent);
        }
// COVEN
	else
		G_SetStats (ent);
	G_CheckChaseStats(ent);

	G_SetClientEvent (ent);

	G_SetClientEffects (ent);

	G_SetClientSound (ent);

	G_SetClientFrame (ent);

	VectorCopy (ent->velocity, ent->client->oldvelocity);
	VectorCopy (ent->client->ps.viewangles, ent->client->oldviewangles);

	// clear weapon kicks
	VectorClear (ent->client->kick_origin);
	VectorClear (ent->client->kick_angles);

	// if the scoreboard is up, update it
	if (ent->client->showscores && !(level.framenum & 31) )
	{
// COVEN:  Update menu or scores.
		if (ent->client->menu)
                        PMenu_Update (ent);
		else
			DeathmatchScoreboardMessage (ent, ent->enemy);
// COVEN
		gi.unicast (ent, false);
	}
}

